@@ -12,11 +12,15 @@
 #'   type. Acceptable responses, and the corresponding error distribution and
 #'   link function used in the \code{glm}, include: \describe{
 #'   \item{binary}{(Default) A binomial distribution with link = 'logit' is
-#'   used.} \item{count}{A Poisson distribution 
-#'   with link = 'log' is used.}
+#'   used.} 
+#'   \item{count}{A Poisson distribution with link = 'log' is used.}
+#'   \item{count_nb}{A negative binomial model with link = 'log' is used, where the theta 
+#'   parameter is estimated internally; ideal for over-dispersed count data.}
 #'   \item{rate}{A Poisson distribution with link = 'log' is used; ideal for 
-#'    events/person-time outcomes.} \item{continuous}{A gaussian distribution 
-#'    with link = 'identity' is used.}
+#'    events/person-time outcomes.} 
+#'    \item{rate_nb}{A negative binomial model with link = 'log' is used, where the theta 
+#'   parameter is estimated internally; ideal for over-dispersed events/person-time outcomes.}
+#'    \item{continuous}{A gaussian distribution with link = 'identity' is used.}
 #'   }
 #' @param formula (Optional) Default NULL. An object of class "formula" (or one
 #'   that can be coerced to that class) which provides the the complete model
@@ -30,8 +34,8 @@
 #'   exposure variable (or treatment group assignment), which can be binary,
 #'   categorical, or continuous. This variable can be supplied as a factor
 #'   variable (for binary or categorical exposures) or a continuous variable.
-#'   For binary/categorical exposures, \code{X} should be supplied as a factor with 
-#'   the lowest level set to the desired referent. Numeric variables are 
+#'   For binary/categorical exposures, \code{X} should be supplied as a factor 
+#'   with the lowest level set to the desired referent. Numeric variables are 
 #'   accepted, but will be centered (see Note). Character variables are not 
 #'   accepted and will throw an error. Can optionally provide a formula
 #'   instead of \code{Y} and \code{X} variables.
@@ -43,17 +47,19 @@
 #'   subgroups for stratified analysis. Effects will be reported for each
 #'   category of the subgroup variable. Variable will be automatically converted
 #'   to a factor if not already.
-#' @param offset (Optional, only applicable for rate outcomes) Default NULL.
-#'   Character argument which specifies the person-time denominator for rate
-#'   outcomes to be included as an offset in the Poisson regression model.
-#'   Numeric variable should be on the linear scale; function will take natural
-#'   log before including in the model.
-#' @param rate.multiplier (Optional, only applicable for rate outcomes) Default
-#'   1. Numeric value to multiply to the rate-based effect measures. This option
-#'   facilitates reporting effects with interpretable person-time denominators.
-#'   For example, if the person-time variable (offset) is in days, a multiplier
-#'   of 365*100 would result in estimates of rate differences per 100
-#'   person-years.
+#' @param offset (Optional, only applicable for rate/count outcomes) Default NULL.
+#'   Character argument which specifies the variable name to be used as the 
+#'   person-time denominator for rate outcomes to be included as an offset in the
+#'   Poisson regression model. Numeric variable should be on the linear scale; 
+#'   function will take natural log before including in the model.
+#' @param rate.multiplier (Optional, only applicable for rate/count outcomes). 
+#'   Default 1 Numeric variable signifying the person-time value to use in 
+#'   predictions; the offset variable will be set to this when predicting under 
+#'   the counterfactual conditions. This value should be set to the person-time 
+#'   denominator desired for the rate difference measure and must be inputted in 
+#'   the units of the original offset variable (e.g. if the offset variable is 
+#'   in days and the desired rate difference is the rate per 100 person-years, 
+#'   rate.multiplier should be inputted as 365.25*100).
 #' @param exposure.scalar (Optional, only applicable for continuous exposure)
 #'   Default 1. Numeric value to scale effects with a continuous exposure. This 
 #'   option facilitates reporting effects for an interpretable contrast (i.e. 
@@ -105,7 +111,16 @@
 #'
 #'   As counterfactual predictions are generated with random sampling of the
 #'   distribution, users should set a seed (\code{\link{set.seed}}) for
-#'   reproducible confidence intervals.
+#'   reproducible confidence intervals. 
+#'   
+#' @note 
+#'  While offsets are used to account for differences in follow-up time 
+#'   between individuals in the \code{glm} model, rate differences are 
+#'   calculated assuming equivalent follow-up of all individuals (i.e. 
+#'   predictions for each exposure are based on all observations having the 
+#'   same offset value). The default is 1 (specifying 1 unit of the original 
+#'   offset variable) or the user can specify an offset to be used in the 
+#'   predictions with the rate.multiplier argument.
 #'
 #' @note
 #'  Note that for a protective exposure (risk difference less than 0), the 
@@ -129,30 +144,34 @@
 #'   exposure. The effects do not necessarily apply across the entire range of 
 #'   the variable. However, variations in the effect are likely small, 
 #'   especially near the mean.
+#'   
+#'  @note 
+#'   For negative binomial models, \code{MASS::glm.nb} is used instead of the
+#'    standard \code{stats::glm} function used for all other models.
 #'     
 #' @references 
 #'  Ahern J, Hubbard A, Galea S. Estimating the effects of potential public health 
 #'   interventions on population disease burden: a step-by-step illustration of 
 #'   causal inference methods. Am. J. Epidemiol. 2009;169(9):1140–1147.
-#'   \href{https://doi.org/10.1093/aje/kwp015}{Manuscript link}
+#'   \doi{10.1093/aje/kwp015}
 #'  
 #'  Altman DG, Deeks JJ, Sackett DL. Odds ratios should be avoided when events 
-#'   are common. BMJ. 1998;317(7168):1318. \href{https://doi.org/10.1136/bmj.317.7168.1318 }{Manuscript link}
+#'   are common. BMJ. 1998;317(7168):1318. \doi{10.1136/bmj.317.7168.1318}
 #' 
 #'  Hernán MA, Robins JM (2020). Causal Inference: What If. Boca Raton: 
-#'   Chapman & Hall/CRC. /href{https://www.hsph.harvard.edu/miguel-hernan/causal-inference-book/}{Book link}
+#'   Chapman & Hall/CRC. \href{https://www.hsph.harvard.edu/miguel-hernan/causal-inference-book/}{Book link}
 #' 
 #'  Robins J. A new approach to causal inference in mortality studies with a 
 #'   sustained exposure period—application to control of the healthy worker 
-#'   survivor effect. Mathematical Modelling. 1986;7(9):1393–1512. \href{https://doi.org/10.1016/0270-0255(86)90088-6}{Manuscript link}
+#'   survivor effect. Mathematical Modelling. 1986;7(9):1393–1512. \doi{10.1016/0270-0255(86)90088-6}
 #'   
 #'  Snowden JM, Rose S, Mortimer KM. Implementation of G-computation on a 
 #'   simulated data set: demonstration of a causal inference technique. 
-#'   Am. J. Epidemiol. 2011;173(7):731–738. \href{https://doi.org/10.1093/aje/kwq472}{Manuscript link}
+#'   Am. J. Epidemiol. 2011;173(7):731–738. \doi{10.1093/aje/kwq472}
 #'   
 #'  Westreich D, Cole SR, Young JG, et al. The parametric g-formula to 
 #'   estimate the effect of highly active antiretroviral therapy on incident 
-#'    AIDS or death. Stat Med. 2012;31(18):2000–2009. \href{https://doi.org/10.1002/sim.5316}{Manuscript link}
+#'    AIDS or death. Stat Med. 2012;31(18):2000–2009. \doi{10.1002/sim.5316}
 #'
 #' @export
 #'
@@ -174,6 +193,7 @@
 #' @importFrom rlang sym
 #' @importFrom magrittr %>%
 #' @importFrom purrr map_dfc
+#' @importFrom MASS glm.nb
 #'
 #' @seealso \code{\link{gComp}}
 
@@ -181,7 +201,7 @@
 
 
 pointEstimate <- function(data, 
-                          outcome.type = c("binary", "count","rate", "continuous"),
+                          outcome.type = c("binary", "count", "count_nb", "rate", "rate_nb", "continuous"),
                           formula = NULL, 
                           Y = NULL, 
                           X = NULL, 
@@ -193,7 +213,7 @@ pointEstimate <- function(data,
                           exposure.center = TRUE) {
 
   # Bind variable locally to function for offset2
-  offset2 <- NULL
+  offset2_name = rlang::sym(paste0(offset, "_adj"))
   
   # Ensure outcome.type is one of the allowed responses
   outcome.type <- match.arg(outcome.type)
@@ -201,17 +221,6 @@ pointEstimate <- function(data,
   # set new df to modify
   working.df <- data
   
-  # Specify model family and link for the given outcome.type
-  if (outcome.type %in% c("binary")) {
-    family <- stats::binomial(link = 'logit')
-  } else if (outcome.type %in% c("count", "rate")) {
-    family <- stats::poisson(link = "log")
-    if (is.null(offset) & outcome.type == "rate") stop("Offset must be provided for rate outcomes")
-  } else if (outcome.type == "continuous") {
-    family <- stats::gaussian(link = "identity")
-  } else {
-    stop("This package only supports binary/dichotomous, count/rate, or continuous outcome variable models")
-  }
   
   # Determine model formula, X, Y, and Z from specified terms, ensure sufficient info provided (either X & Y or formula)
   if (!is.null(X)) X <- rlang::sym(X)
@@ -234,6 +243,25 @@ pointEstimate <- function(data,
     Z <- all.vars(formula[[3]])[-1]
   }
   
+  # Specify model family and link for the given outcome.type
+  if (outcome.type %in% c("binary")) {
+    if (class(working.df[,Y]) == "factor" & length(levels(working.df[,Y])) > 2) stop("Outcome is not binary")
+        family <- stats::binomial(link = 'logit')
+  } else if (outcome.type %in% c("count", "rate")) {
+    if (!is.numeric(working.df %>% dplyr::pull(Y))) stop("Outcome is not numeric")
+    family <- stats::poisson(link = "log")
+    if (is.null(offset) & outcome.type == "rate") stop("Offset must be provided for rate outcomes")
+  } else if (outcome.type %in% c("count_nb", "rate_nb")) {
+    if (!is.numeric(working.df %>% dplyr::pull(Y))) stop("Outcome is not numeric")
+    family = NULL
+    if (is.null(offset) & outcome.type == "rate_nb") stop("Offset must be provided for rate outcomes")
+  } else if (outcome.type == "continuous") {
+    if (!is.numeric(working.df %>% dplyr::pull(Y))) stop("Outcome is not numeric")
+    family <- stats::gaussian(link = "identity")
+  } else {
+    stop("This package only supports binary/dichotomous, count/rate, or continuous outcome variable models")
+  }
+  
   # Determine X_mean
   if (is.numeric(data[[X]])) X_mean = ifelse(exposure.center == T, round(mean(data[[X]]), 2), round(exposure.center, 2))
   
@@ -244,23 +272,16 @@ pointEstimate <- function(data,
   }
   
   # Make list of variables that will be used (Y, X, Z, and subgroup/offset if specified)
-  if (is.null(offset) & is.null(subgroup)) {
-    allVars <- unlist(c(Y, as.character(X), Z))
-  } else if (!is.null(offset)) {
+  
+  allVars <- unlist(c(Y, as.character(X), Z))
+  if (!is.null(offset)) {
     offset <- rlang::sym(offset)
-    # data <- data %>%
-    #   dplyr::mutate(offset2 = !!offset + 0.00001,
-    #                 logOffset = log(offset2))
-    if (!is.null(subgroup)){
-      subgroup <- rlang::sym(subgroup)
-      allVars <- unlist(c(Y, as.character(X), Z, offset, subgroup))
-    } else {
-      allVars <- unlist(c(Y, as.character(X), Z, offset))
-    }
-  } else {
-    subgroup <- rlang::sym(subgroup)
-    allVars <- unlist(c(Y, as.character(X), Z, subgroup))
+    allVars <- c(allVars, as.character(offset))
   }
+  if (!is.null(subgroup)) {
+    subgroup <- rlang::sym(subgroup)
+    allVars <- c(allVars, subgroup)
+  } 
   
   # Ensure all variables are provided in the dataset
   if (!all(allVars %in% names(working.df))) stop("One or more of the supplied model variables, offset, or subgroup is not included in the data")
@@ -298,14 +319,24 @@ pointEstimate <- function(data,
   # Run GLM
   if (!is.null(offset)) {
     working.df <- working.df %>%
-      dplyr::mutate(offset2 = !!offset + 0.00001)
-    glm_result <- stats::glm(formula = formula, data = working.df, family = family, na.action = stats::na.omit, offset = log(offset2))
-  } else {
+      dplyr::mutate(!!offset2_name := !!offset + 0.00001) 
+    formula <- stats::as.formula(paste(paste0(as.character(formula)[2], as.character(formula)[1], as.character(formula)[3]), paste0("offset(log(", offset2_name, "))"), sep = " + "))
+    if (!outcome.type %in% c("count_nb", "rate_nb")) {
+      glm_result <- stats::glm(formula = formula, data = working.df, family = family, na.action = stats::na.omit)
+      # eval(parse(text = paste0(
+      #   "glm_result <- stats::glm(formula = formula, data = working.df, family = family, na.action = stats::na.omit, offset = log(",offset2_name,"))"
+      # )))
+    } else if (outcome.type %in% c("count_nb", "rate_nb")) {
+      glm_result <- MASS::glm.nb(formula = formula, data = working.df, na.action = stats::na.omit)
+    }
+  } else if (!outcome.type %in% c("count_nb", "rate_nb")) {
     glm_result <- stats::glm(formula = formula, data = working.df, family = family, na.action = stats::na.omit)
+  } else {
+    glm_result <- MASS::glm.nb(formula = formula, data = working.df, na.action = stats::na.omit)
   }
   
   # Predict outcomes for each observation/individual at each level of treatment/exposure
-  fn_output <- make_predict_df(glm.res = glm_result, df = working.df, X = X, subgroup = subgroup, offset = offset)
+  fn_output <- make_predict_df(glm.res = glm_result, df = working.df, X = X, subgroup = subgroup, offset = offset, rate.multiplier = rate.multiplier)
   
   # Rename vars in predicted dataset so it's clear
   results_tbl_all <- fn_output
@@ -332,7 +363,7 @@ pointEstimate <- function(data,
         subgroup_res <- suppressMessages(purrr::map_dfc(subgroups_list, function(s) {
           predict_df_s = fn_output %>% 
             dplyr::select(tidyselect::contains(s))
-          fn_results_df <- get_results_dataframe(predict.df = predict_df_s, outcome.type = outcome.type, rate.multiplier = rate.multiplier)
+          fn_results_df <- get_results_dataframe(predict.df = predict_df_s, outcome.type = outcome.type)
           return(c(fn_results_df, subgroup = s, Comparison = paste0(e, "_v._", exposure_list[1])))
         }))
         subgp_results <- subgroup_res %>%
@@ -346,7 +377,7 @@ pointEstimate <- function(data,
         # s <- subgroups_list[1]
         predict_df_s = fn_output %>% 
           dplyr::select(tidyselect::contains(s))
-        fn_results_df <- get_results_dataframe(predict.df = predict_df_s, outcome.type = outcome.type, rate.multiplier = rate.multiplier)
+        fn_results_df <- get_results_dataframe(predict.df = predict_df_s, outcome.type = outcome.type)
         return(c(fn_results_df, subgroup = s, Comparison = contrasts_list))
       }))
       results <- subgroup_res %>%
@@ -358,14 +389,14 @@ pointEstimate <- function(data,
       # e <- exposure_list[2]
       predict_df_e <- fn_output %>%
         dplyr::select(tidyselect::contains(match = c(exposure_list[1], e))) #contains(exposure_list[1]), tidyselect::contains(e))
-      fn_results_df <- get_results_dataframe(predict.df = predict_df_e, outcome.type = outcome.type, rate.multiplier = rate.multiplier)
+      fn_results_df <- get_results_dataframe(predict.df = predict_df_e, outcome.type = outcome.type)
       return(c(fn_results_df, subgroup = NA, Comparison = paste0(e, "_v._", exposure_list[1])))
     }))
     results <- contrasts_res %>%
       as.data.frame()
     colnames(results) <- contrasts_list
   } else { # For when NO subgroups are specified and exposure has only 2 levels
-    fn_results_df <- get_results_dataframe(predict.df = fn_output, outcome.type = outcome.type, rate.multiplier = rate.multiplier)
+    fn_results_df <- get_results_dataframe(predict.df = fn_output, outcome.type = outcome.type)
     
     results <- fn_results_df %>%
       as.data.frame() %>%
